/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2008 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.launcher;

import java.io.IOException;

import javax.naming.ConfigurationException;
import javax.xml.bind.JAXBException;

import org.objectweb.fractal.adl.ADLException;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.ow2.petals.PetalsException;
import org.ow2.petals.kernel.server.PetalsListener;
import org.ow2.petals.kernel.server.PetalsServer;
import org.ow2.petals.launcher.util.CommandReader;
import org.ow2.petals.launcher.util.SystemExitHook;
import org.ow2.petals.tools.jmx.api.PetalsJMXClient;
import org.ow2.petals.tools.jmx.api.exception.ConnectionErrorException;
import org.ow2.petals.tools.jmx.api.exception.PetalsAdminDoesNotExistException;
import org.ow2.petals.tools.jmx.api.exception.PetalsAdminServiceErrorException;
import org.ow2.petals.topology.TopologyException;

/**
 * 
 * Created on 29 janv. 08
 * 
 * @author Christophe HAMERLING - eBM WebSourcing
 * @since
 * 
 */
public abstract class AbstractLauncher implements PetalsListener {
	protected static final String START_COMMAND = "start";

	protected static final String STOP_COMMAND = "stop";

	protected static final String SHUTDOWN_COMMAND = "shutdown";

	protected static final String VERSION_COMMAND = "version";

	/**
	 * The Petals server instance
	 */
	protected PetalsServer petalsServer;

	/**
	 * System exit hook used if the command line is used
	 */
	protected SystemExitHook systemExitHook;

	/**
	 * 
	 * 
	 */
	public AbstractLauncher() {
	}

	/**
	 * 
	 * @param args
	 */
	public void launch(String[] args) {

		// Process the Command and exits abnormaly if necessary

		String command = null;
		boolean explore = false;
		boolean console = false;

		try {
			// launcher = new PlatformLauncher();

			if (args.length == 0) {
				System.err
						.println("PetalsPlatformLauncher usage: start|stop|shutdown [-console] [-explorer]");
				System.err.println("console : activate the console mode");
				System.err.println("explorer : activate the FRACTAL explorer");
				System.exit(-1);
			}

			for (String arg : args) {
				if ("-explore".equals(arg)) {
					explore = true;
				} else if ("-console".equals(arg)) {
					console = true;
				} else {
					command = arg;
				}
			}

			if (STOP_COMMAND.equals(command)) {
				System.out.println("Petals is stopping...");
				PetalsJMXClient petalsJmxClient = getPetalsJmxClient();
				petalsJmxClient.getPetalsAdminServiceClient().stopContainer();
			} else if (SHUTDOWN_COMMAND.equals(command)) {
				System.out.println("Petals is shutting down...");
				PetalsJMXClient petalsJmxClient = getPetalsJmxClient();
				petalsJmxClient.getPetalsAdminServiceClient()
						.shutdownContainer();
			} else if (VERSION_COMMAND.equals(command)) {
				PetalsJMXClient petalsJmxClient = getPetalsJmxClient();
				System.out.println(petalsJmxClient.getAdminServiceClient()
						.getSystemInfo());
			} else if (START_COMMAND.equals(command)) {

				petalsServer = new PetalsServer();

				// show the fractal explorer mode if asked
				if (explore) {
					petalsServer.setObserver(true);
				}

				// check if started or not stopped correctly
				if (petalsServer.isLocked()) {
					throw new PetalsException(
							"Can not start the PEtALS server, remove lock file or stop server");
				}

				petalsServer.addStopListener(this);

				// add a hook if a terminate signal is sent from the command
				// line
				systemExitHook = new SystemExitHook(petalsServer);
				Runtime.getRuntime().addShutdownHook(systemExitHook);

				System.out.println("Petals is starting...");
				petalsServer.init(false);
				petalsServer.start();
				
				// print the system info
				PetalsJMXClient client = getPetalsJmxClient();
				System.out.println("System Info : " + client.getAdminServiceClient().getSystemInfo());
				System.out.println();
				
				// show the commandLine mode if asked
				if (console) {
					commandLineMode();
				}
			} else {
				System.err.println("Command '" + command + "' unknown");
				System.exit(-1);
			}
		} catch (Throwable e) {
			System.err.println("Command processing error: " + command);
			e.printStackTrace();
			System.exit(-1);
		}
	}

	/**
	 * 
	 * @return
	 * @throws IOException
	 * @throws JAXBException
	 * @throws TopologyException
	 * @throws ConnectionErrorException
	 * @throws PetalsAdminDoesNotExistException
	 * @throws PetalsAdminServiceErrorException
	 * @throws ConfigurationException
	 */
	protected abstract PetalsJMXClient getPetalsJmxClient() throws IOException,
			JAXBException, TopologyException, ConnectionErrorException,
			PetalsAdminDoesNotExistException, PetalsAdminServiceErrorException,
			ConfigurationException;

	/**
	 * Show a command line to interact with Petals. TODO we pass the PetalsAdmin
	 * object to the CommandReader to perform some actions. Find a better way.
	 * 
	 * @throws TopologyException
	 * @throws JAXBException
	 * @throws IOException
	 * @throws PetalsAdminServiceErrorException
	 * @throws PetalsAdminDoesNotExistException
	 * @throws ConnectionErrorException
	 * @throws ConfigurationException
	 * @throws NoSuchInterfaceException
	 * @throws ADLException
	 * @throws PetalsException
	 */
	protected void commandLineMode() throws ConnectionErrorException,
			PetalsAdminDoesNotExistException, PetalsAdminServiceErrorException,
			IOException, JAXBException, TopologyException,
			ConfigurationException {
		PetalsJMXClient petalsJmxClient = getPetalsJmxClient();
		CommandReader console = new CommandReader(petalsJmxClient
				.getPetalsAdminServiceClient(), petalsServer);
		console.read();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.objectweb.petals.kernel.server.PetalsListener#onPetalsStarted()
	 */
	public void onPetalsStarted() {
		System.out.println("##### Petals container correctly started #####");
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.objectweb.petals.kernel.server.PetalsListener#onPetalsStopped()
	 */
	public void onPetalsStopped() {
		// remove the System Hook to avoid the Petals server stop() called a
		// second time
		Runtime.getRuntime().removeShutdownHook(systemExitHook);
		System.out.println("Petals is stopped");

		System.exit(0);
	}
}
